%    Alg8.m
function [x,xB,nB,IB,fB,HBinv,msg,dual,iter,alg] = ...
                       Alg8(c,C,n,m,tol,xB,IB,nB,A,b)  
%   Alg8 implements the Simplex Method for Quadratic
%   Programming.  It uses Procedures Psi1, Psi2 and Psi3
%   to do the matrix upating.
%   Assume xB,IB and nB all have the correct values
%   on input.
%   Initialzation
[ B,cB,CBB,gB,gN,fB,IN ] = simpobj(A,C,c,n,xB,IB,nB);
HB = vertcat([CBB,B'],[B,zeros(m,m)]);
HBinv = inv(HB);
alg = 'Algorithm 8 (Simplex Method for QP)';
next = 'Step1';
singular = 'false';
for iter=0:10000
    iteration = iter;
    if strcmp(next,'Step1')
                             % Step 1
        if strcmp(singular,'false')
            next = 'Step1p1';
        else
            next = 'Step1p2';
        end
    end
    if strcmp(next,'Step1p1')
                             % Step 1.1
        rhs = vertcat(gB,zeros(m,1));
        sBu = HBinv*rhs;
        sB = sBu(1:nB);
        u = sBu(nB+1:nB+m);
        gammaB = 0;
        next = 'Step2';
    end    
    if strcmp(next,'Step1p2')
                             % Step 1.2
        sB =  w(1:nB);
        gammaB = 1;
        next = 'Step2';
    end
    if strcmp(next,'Step2')
                             % Step 2
        if gammaB == 0
            sigBtilde = 1;
        else
            sigBtilde = inf;
        end
        sigBhat = inf;
        for i=1:nB
            if sB(i) > 0.
                test = xB(i)/sB(i);
                if test < sigBhat
                    sigBhat = test;
                    ell = i;
                end
            end
        end
        if sigBtilde == inf & sigBhat == inf
            msg = 'unbounded from below';
            return
        else
            sigB = min(sigBtilde,sigBhat);
        end
    next = 'Step3';
    end    
    if strcmp(next,'Step3')
                             % Step 3
        if sigB == sigBhat
            next = 'Step3p1';
        else
            next = 'Step3p2';
        end
        if strcmp(next,'Step3p1')
                             % Step 3.1
            if strcmp(singular,'false')
                xB = xB - sigB*sB;
                xB(ell) = [];
                [ HBinv,singular,w ] = Psi2(HBinv,m+nB,ell);
                IB(ell) = [];
                nB = nB - 1;
                [ B,cB,CBB,gB,gN,fB,IN ] = ...
                       simpobj(A,C,c,n,xB,IB,nB);
                next = 'Step1';
            end
            
            if strcmp(singular,'true')
                xB = xB -sigB*sB;
                xB(ell) = sigB;
                d = [];
                for i=1:ell-1
                    d(i) = C(IB(i),kay) - C(IB(i),IB(ell));
                end
                d(ell) = (C(kay,kay) - C(IB(ell),IB(ell)))/2.;
                for i=ell+1:nB
                    d(i) = C(IB(i),kay) - C(IB(i),IB(ell));
                end
                for i=1:m
                    d(i+nB) = A(i,kay) - A(i,IB(ell));
                end
                d  = d';
                [ HBinv,singular,w ] = ... 
                           Psi3( HBinv,d,nB+m,ell );
                IB(ell) = kay;
                [ B,cB,CBB,gB,gN,fB,IN ] = ...
                       simpobj(A,C,c,n,xB,IB,nB);
                next = 'Step1';
            end
        end    
        if strcmp(next,'Step3p2')
                             % Step 3.2
            xB = xB - sigB*sB;
            [ B,cB,CBB,gB,gN,fB,IN ] = ...
                   simpobj(A,C,c,n,xB,IB,nB);
            u = - u;
            for i=1:nB
                v(IB(i)) = 0.;
            end
            for i=1:n-nB
                j = IN(i);
                v(j) = A(:,j)'*u + gN(i);
            end    
            smallv = min(v);
            kay = find(smallv==v,1);
            if smallv >= - tol;
                msg = 'Optimal solution obtained';
                x = zeros(n,1);
                for i=1:nB
                    x(IB(i)) = xB(i);
                end 
                dual = vertcat(v',u);
                return
            end 
            d1 = [];
            for i=1:nB
                d1(i) = C(IB(i),kay);
            end
            d1 = d1';
            [HBinv,singular,w] = ...
                   Psi1(HBinv,d1,C(kay,kay),A(:,kay),m+nB,nB);
             if strcmp(singular,'false')
                 xB = vertcat(xB,0);
         %        xB(nB+1) =0.;
                 IB(nB+1) = kay;
                 nB = nB + 1;
                 kpos = find(kay==IN,1);
                 gB = vertcat(gB,gN(kpos));
                 gN(kpos) = [];
                 next = 'Step1';
             end
             if strcmp(singular,'true')
                 next = 'Step1';
             end
        end         
    end    
end       % end of for iter loop

