%    Psi1.m
function [ H1inv,singular,w ] = Psi1 ( H0inv,d1,gamma,d2,p,q )

%Psi1 implements Procedure Psi 1 of Chapter 7, text.
%A matrix with a known inverse (H0inv) is augmented with a 
%new row and column (d1,gamma,d2) and Procedure Psi1 finds
%inverse (H1inv) of the new matrix.  If the new matrix is
%singular, Psi1 returns a vector w in its null space.
%
d = vertcat(d1,d2);  
tol = 1.e-6;
if q >=  1 & q < p
    theta = gamma - d' * H0inv * d;
    if abs(theta) <= tol                 % H1 is singular
        singular = 'true';
        dtilde = H0inv * d;  
        dtilde1 = dtilde(1:q);   dtilde2 = dtilde(q+1:p);
        w = vertcat(d1,-1,d2);   % w is in null space of H1
        H1inv = [];
        return
        
    else                                % H1 is nonsingular
        singular = 'false';
        w = [];
        beta = 1./theta;
        temp = H0inv * d;
        B = H0inv + beta * (temp * temp');
        e = - beta * temp;  e1 = e(1:q);  e2 = e(q+1:p);
        B11 = B(1:q,1:q);       B12 = B(1:q,q+1:p);  
        B21 = B(q+1:p,1:q);     B22 = B(q+1:p,q+1:p);
        H1inv = vertcat([B11,e1,B12],[e1',beta,e2'], ...
                                     [B21,e2,B22]);
        return                         
    end
end    

if q == 0
    theta = gamma - d2'*H0inv*d2;
    if abs (theta) < tol
        singular = 'true';                % H1 is singular 
        d2tilde = H0inv*d2;              
        w = vertcat(-1,d2tilde);
        H1inv = [];
    else                                % H1 is nonsingular
        singular = 'false';
        w = [];
        beta = 1./theta;
        B22 = H0inv + beta*H0inv*d2*d2'*H0inv;
        e2 = - beta*H0inv*d2;
        H1inv = vertcat([beta,e2'],[e2,B22]);
        return
    end    
end

if q == p
    theta = gamma - d1'*H0inv* d1;
    if abs(theta) < tol
        singular = 'true';                 % H1 is singular
        d1tilde = H0inv*d1;
        w = vertcat(d1tilde,-1);
        H1inv = [];
        return
    else                                
        singular = 'false';                % H1 is nonsingular
        w = [];
        beta = 1./theta;
        B11 = H0inv + beta*H0inv*d1*d1'*H0inv;
        e1 = -beta*H0inv*d1;
        H1inv = vertcat([B11,e1],[e1',beta]);
        return
    end
end

return
