%    Psi2.m
function [ H1inv,singular,w ] = Psi2 ( H0inv,p,q )

%Psi2 implements Procedure Psi 2 of Chapter 7, text.
%A matrix with a known inverse (H0inv) has a row and  
% column deleted from it. Procedure Psi2 finds
%inverse (H1inv) of the new matrix.  If the new matrix is
%singular, Psi2 returns a vector w in its null space.
%
tol = 1.e-6;
if q > 1 & q < p
    beta = H0inv(q,q);
    if abs(beta) <= tol                % H1 is singular
        singular = 'true';
        e1 = H0inv(1:q-1,q);
        e2 = H0inv(q+1:p,q);
        w = vertcat(e1,e2);            % w is in null(H1)
        H1inv = [];
        return
        
    else                               % H1 is nonsingular
        singular = 'false';
        w = [];
        e1 = H0inv(1:q-1,q);
        e2 = H0inv(q+1:p,q);
        e = vertcat(e1,e2);
        B11 = H0inv(1:q-1,1:q-1);  B12 = H0inv(1:q-1,q+1:p);
        B21 = H0inv(q+1:p,1:q-1);  B22 = H0inv(q+1:p,q+1:p);
        H1inv = vertcat( [B11,B12],[B21,B22] ) ...
                                - 1./beta*e*e' ;
        return
    end
end

if q == 1
    beta = H0inv(1,1);
    if abs(beta) <= tol                % H1 is singular  
        singular = 'true';
        e2 = H0inv(2:p,1);
        w = e2;                        % w is in null(H1)
        H1inv = [];
    else                               % H1 is nonsingular
        singular = 'false';
        w = [];
        B22 = H0inv(2:p,2:p);  e2 = H0inv(2:p,1);
        H1inv = B22 - (1./beta)*e2*e2';
        return
    end
end

if q == p
    beta = H0inv(p,p)
    if abs(beta) <= tol                % H1 is singular
        singular = 'true';
        e1 = H0inv(1:p-1,p);
        w = e1;                        % w is in null(H1)
        H1inv = [];
    else                               % H1 is nonsingular
        singular = 'false';
        w = [];
        B11 = H0inv(1:p-1,1:p-1)
        e1 = H0inv(1:p-1,p)
        H1inv = B11 - (1./beta)*e1*e1'
        return
    end
end
    