%  checkKKT.m
function checkKKT(A,b,x,n,m,r,dual,c,C,fx,j,msg,alg)
%  Given a proposed optimal solution for a QP,
%  checkKKT evaluates the Karush-Kuhn-Tucker conditions
%  at x.
if strcmp(msg,'Unbounded from below');
    iter = j;
    fprintf('Optimization Summary\n');
    fprintf('%s\n',msg);
    fprintf('Algorithm: %s\n',alg);
    fprintf('Iterations   %d\n',iter);
    return
end    
iter = j;
w1 = 1.;
eqnorm = 0.0;
for i=1:m
    sum = - b(i);
    for j=1:n
        sum  = sum + A(i,j)*x(j);
    end
    y(i) = sum;
end
ineqmax = max(y);
if r >= 1
    for i=m+1:m+r
        sum = - b(i);
        for j=1:n
            sum  = sum + A(i,j)*x(j);
        end
        z(i) = sum;
    end
    eqnorm = norm(z);
end

grad = c + C*x;
for j=1:m+r
    if abs(dual(j)) >=  1.e-6
        grad = grad + dual(j)*A(j,:)';
    end
end
dfnorm = norm(grad);

primalerr = max(ineqmax,eqnorm);



fprintf('Optimization Summary\n');
fprintf('%s\n',msg);
fprintf('Algorithm: %s\n',alg);
fprintf('Optimal objective value   %.8f\n',fx);
fprintf('Iterations   %d\n',iter);
fprintf('Maximum Primal Error      %.8f\n',primalerr);
fprintf('Maximum Dual Error        %.8f\n',dfnorm);

fprintf('\n\n Optimal Primal Solution\n')
for i=1:n
    fprintf('   %d %12.8f\n',i,x(i))
end
fprintf('\n\n Optimal Dual Solution\n')
for i=1:m+r
    fprintf('   %d %12.8f\n',i,dual(i))
end
